/*
% Název BP: 
% CS: Analýza zakřivení páteře s využitím moderních senzorů
% EN: Back-bone deformation analysis using modern sensors

% Kód pro funkci příjem a vizualizaci dat zasílaných přes bluetooth ze zařízení
% Autor: Vojtěch Fiala
% Datum: 26. 4. 2018

-----------------------------------------------------------------------------------------------------------------
Tento kód vznikl vhodným sestavením a odladěním výše zmíněným autorem. Původní kódy, ze kterých bylo čerpáno, jsou
uvedeny ve zdrojích v rámci textu k BP, k němuž je přiložen i tento program. Tento kód byl sestaven pro danou práci
a zařízení, které bylo v rámci ní sestrojeno.
*/
 
 /*
 *  Program for visualizing rotation data about the three IMU axes as calculated by the MPU6050 DMP
 *  and the complementary filter.
 */

// Prvotní inicializace proměnných a rozměrů
import processing.serial.*;

Serial  myPort;
short   portIndex = 1; // Index of serial port in list (varies by computer)
int     lf = 10;       //ASCII linefeed
String  inString;      //String for testing serial communication

// x,y coordinates of circles 1, 2, 3 (pitch, roll, yaw)
int cx[] = {150, 500};
int cy[] = {160, 360, 560};

// circle diameters
int d   = 150; 

// Data from dmp and complementary filters
float dmp[] = new float[2];
float cmp[] = new float[2];
float lmp[] = new float[2];


/*
 * Draws a line of length len, centered at x, y at the specified angle
 */
void drawLine(int x, int y, int len, float angle) {
  pushMatrix();
  translate(x, y);
  rotate(angle);
  line(-len/2, 0, len/2, 0);
  popMatrix();
}


//===============================
// Klasický SETUP
//===============================
void setup() {
  
  // Set up the main window
  size(650, 680);
  //background(0);
  
  // Set up serial port access
  //  println("in setup");
  // String portName = Serial.list()[portIndex];
  
  
  // Nutno upravit podle portu, na kterém komunikuje bluetooth s PC
  //==============================================================
  String portName = "COM13";
  //==============================================================
  
  //  println(Serial.list());
  //  println(" Connecting to -> " + Serial.list()[portIndex]);
  myPort = new Serial(this, portName, 38400);
  myPort.clear();
  myPort.bufferUntil(lf);
}


//========================================
// Opakující se smyčka programu
//========================================
void draw() {
  
  background(0);
  
  // Draw the three background circles
  noStroke();
  fill(225);
  for (int i = 0; i < 3; i++) {
    for (int j = 0; j < 2; j++) {
      ellipse(cx[j], cy[i], d, d);
    }
  }
  
  // Draw the lines representing the angles
  for (int i = 1; i >= 0; i--) {
    strokeWeight(3);
    stroke(255, 0, 0);
    drawLine(cx[1-i], cy[0], d, radians(cmp[i])); // Krční senzor
    stroke(255, 0, 0);
    drawLine(cx[1-i], cy[1], d, radians(dmp[i])); // Hrudní senzor
    stroke(255, 0, 0);
    drawLine(cx[1-i], cy[2], d, radians(lmp[i])); // Bederní senzor
  }
  
  // Draw the explanatory text
  textSize(25);
  fill(255, 0, 0);
  text("Analýza zakřivení páteře", 178, 30);
  
  textSize(20);
  fill(#2EBDF0);
  text("Krční senzor", 268, cy[0]);
  fill(#2EBDF0);
  text("Hrudní senzor", 258, cy[1]);
  fill(#2EBDF0);
  text("Bederní senzor", 254, cy[2]);
  
  fill(255);
  text("Předozadní sklon", cx[0]-88, 70);
  text("Boční sklon", cx[1]-63, 70);
 
  textSize(18);
  for (int i = 1; i >= 0; i--) {
    fill(255, 0, 0);
    String str = String.format("%6.2f", cmp[i]);
    text(str, cx[1-i]-30, cy[0]+100);
    fill(255, 0, 0);
    str = String.format("%6.2f", dmp[i]);
    text(str, cx[1-i]-30, cy[1]+100);
    fill(255, 0, 0);
    str = String.format("%6.2f", lmp[i]);
    text(str, cx[1-i]-30, cy[2]+100);
  }
  
}


/*
 *  Read and process data from the serial port
 */
void serialEvent(Serial p) {
  inString = myPort.readString();
  
  try {
    // Parse the data
    //println(inString);
    String[] dataStrings = split(inString, ':');
    if (dataStrings.length == 3) {
      if (dataStrings[0].equals("1sen")) {
        for (int i = 0; i < dataStrings.length - 1; i++) {
          cmp[i] = float(dataStrings[i+1]); // Krční senzor
        }
      } else if (dataStrings[0].equals("2sen")) {
        for (int i = 0; i < dataStrings.length - 1; i++) {
          dmp[i] = float(dataStrings[i+1]); // Hrudní senzor
        }        
      } else if (dataStrings[0].equals("3sen")) {
        for (int i = 0; i < dataStrings.length - 1; i++) {
          lmp[i] = float(dataStrings[i+1]); // Bederní senzor
        }        
      } else {
        println(inString);
      }
    }
  } catch (Exception e) {
    println("Caught Exception");
  }
  
}
